package betterwithmods.common.blocks;

import betterwithmods.common.BWMBlocks;
import betterwithmods.common.tile.TileVase;
import betterwithmods.util.CapabilityUtils;
import betterwithmods.util.ColorUtils;
import betterwithmods.util.InvUtils;
import com.google.common.collect.Maps;
import net.minecraft.block.Block;
import net.minecraft.block.SoundType;
import net.minecraft.block.state.BlockFaceShape;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.projectile.EntityArrow;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.EnumDyeColor;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumHand;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.RayTraceResult;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;

import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Random;

/**
 * Created by Christian on 24.09.2016.
 */
public class BlockVase extends BWMBlock {
    public static final HashMap<EnumDyeColor, Block> BLOCKS = Maps.newHashMap();
    private static final AxisAlignedBB AABB = new AxisAlignedBB(0.125D, 0, 0.125D, 0.875D, 1.0D, 0.875D);

    public BlockVase(EnumDyeColor color) {
        super(BWMBlocks.POTTERY);
        this.func_149711_c(2.0F);
        this.setHarvestLevel("pickaxe", -1);
        this.func_149672_a(SoundType.field_185853_f);
        this.setRegistryName("vase_" + color.func_176610_l());
    }

    public static void init() {
        for (EnumDyeColor color : ColorUtils.DYES) {
            BLOCKS.put(color, new BlockVase(color));
        }
    }

    public static ItemStack getStack(EnumDyeColor type) {
        return new ItemStack(BLOCKS.get(type));
    }

    @Override
    public boolean hasTileEntity(IBlockState state) {
        return true;
    }

    @Override
    public TileEntity createTileEntity(@Nonnull World world, @Nonnull IBlockState state) {
        return new TileVase();
    }

    @Nonnull
    @SuppressWarnings("deprecation")
    @Override
    public AxisAlignedBB func_185496_a(IBlockState state, IBlockAccess world, BlockPos pos) {
        return AABB;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149662_c(IBlockState state) {
        return false;
    }

    @SuppressWarnings("deprecation")
    @Override
    public boolean func_149686_d(IBlockState state) {
        return false;
    }

    @Override
    public void func_180633_a(World world, BlockPos pos, IBlockState state, EntityLivingBase placer, ItemStack stack) {
        TileEntity tile = world.func_175625_s(pos);
        CapabilityUtils.getInventory(tile, null).ifPresent(inv -> InvUtils.readFromStack(inv, stack));
    }

    @Override
    public boolean func_180639_a(World worldIn, BlockPos pos, IBlockState state, EntityPlayer playerIn, EnumHand hand, EnumFacing side, float hitX, float hitY, float hitZ) {
        ItemStack heldItem = playerIn.func_184586_b(hand);
        if (playerIn.func_70093_af())
            return false;
        InvUtils.getItemHandler(worldIn, pos, null).ifPresent(inv -> {
            if (InvUtils.insertSingle(inv, heldItem, false).func_190926_b()) {
                if (!playerIn.func_184812_l_())
                    heldItem.func_190918_g(1);
                worldIn.func_184148_a(null, pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p(),
                        SoundEvents.field_187638_cR, SoundCategory.PLAYERS, 0.2F,
                        ((worldIn.field_73012_v.nextFloat() - worldIn.field_73012_v.nextFloat()) * 0.7F + 1.0F) * 2.0F);
            }
        });
        return super.func_180639_a(worldIn, pos, state, playerIn, hand, side, hitX, hitY, hitZ);
    }

    @Override
    public void func_180634_a(World world, BlockPos pos, IBlockState state, Entity entity) {
        if (!world.field_72995_K && entity instanceof EntityArrow) {
            world.func_175718_b(2001, pos, Block.func_176210_f(state));
            world.func_175666_e(pos, this);
            world.func_175698_g(pos);
        }
        super.func_180634_a(world, pos, state, entity);
    }

    @Nonnull
    @Override
    public ItemStack getPickBlock(@Nonnull IBlockState state, RayTraceResult target, @Nonnull World world, @Nonnull BlockPos pos, EntityPlayer player) {
        ItemStack stack = new ItemStack(this);
        TileEntity tile = world.func_175625_s(pos);
        CapabilityUtils.getInventory(tile, null).ifPresent(inv -> InvUtils.writeToStack(inv, stack));
        return stack;
    }

    @Override
    public boolean canSilkHarvest(World world, BlockPos pos, @Nonnull IBlockState state, EntityPlayer player) {
        return true;
    }

    public boolean func_149740_M(IBlockState state) {
        return true;
    }

    @Override
    public int func_149745_a(Random random) {
        return 0;
    }

    @SuppressWarnings("deprecation")
    @Nonnull
    @Override
    public BlockFaceShape func_193383_a(IBlockAccess worldIn, IBlockState state, BlockPos pos, EnumFacing face) {
        return face == EnumFacing.DOWN ? BlockFaceShape.CENTER_BIG : (face == EnumFacing.UP ? BlockFaceShape.CENTER : BlockFaceShape.UNDEFINED); //Top is center instead of bowl to facilitate placing candles on these.
    }
}
